import os
import logging
from typing import Union
from jinja2 import Environment, FileSystemLoader
logger = logging.getLogger(__name__)

default_plugins_to_update = [
    "ascanrules",
    "ascanrulesAlpha",
    "ascanrulesBeta",
    "pscanrulesBeta",
    "pscanrulesAlpha",
    "automation",
    "domxss",
    "graphql",
    "openapi",
    "reflect",
    "reports",
    "soap",
    "spiderAjax"
]


class AddOnsConfig:
    def __init__(self, update_plugins: bool = True, plugins_to_update: list = None):
        if update_plugins:
            self.update_plugins_string = "true"
        else:
            self.update_plugins_string = "false"

        if plugins_to_update:
            self.plugins_to_update = plugins_to_update
        # Case: If no value is supplied at all, use the default list
        elif isinstance(plugins_to_update, type(None)):
            self.plugins_to_update = default_plugins_to_update
        # Case: It is an empty list
        elif isinstance(plugins_to_update, list):
            self.plugins_to_update = []


class ActiveScanConfig:
    def __init__(self, active_scan_rule_ids: Union[list, str] = None, strength: str = "Medium", threshold: str = "Medium", default_strength: str = "Medium", default_threshold: str = "Off"):
        self.active_scan_rule_ids = get_rule_id_list(active_scan_rule_ids)
        # Rule specific strength and alert level
        self.alert_threshold = self._validate_threshold(threshold=threshold, var_name="threshold")
        self.attack_strength = self._validate_strength(strength=strength, var_name="strength")

        # Default strength and alert level for all rules. This will make the scan look for all other alerts, not just the specified ones - so the results will be quite noisy.
        self.default_threshold = self._validate_threshold(threshold=default_threshold, var_name="default_threshold")
        self.default_strength = self._validate_strength(strength=default_strength, var_name="default_strength")
        if os.getenv("UNIT_TEST"):
            print("NOTE: found `UNIT_TEST` environment variable. Setting max_rule_duration and max_scan_duration to 1 (the lowest possible value) so the test scan length is reduced. This will improve development experience.")
            self.max_rule_duration = 1
            self.max_scan_duration = 1
        else:
            logger.info("NOTE: `UNIT_TEST` environment variable was not found. Setting max_rule_duration and max_scan_duration to 10 so we can fit the scans inside of the Lambda 15 minute invocation limit.")
            self.max_rule_duration = 10
            self.max_scan_duration = 10

    def _validate_strength(self, strength: str, var_name: str):
        allowed_values = ["Low", "Medium", "High", "Insane"]
        if strength not in allowed_values:
            raise Exception(f"The {var_name} level is set to {strength}, which is an invalid value. It should be one of: {','.join(allowed_values)}")
        else:
            return strength

    def _validate_threshold(self, threshold: str, var_name: str):
        allowed_values = ["Off", "Low", "Medium", "High"]
        if threshold not in allowed_values:
            raise Exception(f"The {var_name} level is set to {threshold}, which is an invalid value. It should be one of: {','.join(allowed_values)}")
        else:
            return threshold


class ReportConfig:
    def __init__(self):
        self.template_id = "traditional-json"
        self.report_dir = "/tmp/zap/"  # nosec


class ZapAutomationTemplate:
    def __init__(self, target_url: str, include_paths: str = "", exclude_paths: str = "", update_plugins: bool = True,
                 plugins_to_update=None, strength: str = "Medium", threshold: str = "Medium", default_strength: str = "Medium", default_threshold: str = "Off"):
        if plugins_to_update is None:
            plugins_to_update = default_plugins_to_update
        self.target_url = target_url
        self.include_paths = include_paths
        self.exclude_paths = exclude_paths
        self.update_plugins = update_plugins
        self.plugins_to_update = default_plugins_to_update
        self.add_ons = AddOnsConfig(update_plugins=self.update_plugins, plugins_to_update=plugins_to_update)
        self.alert_threshold = threshold
        self.attack_strength = strength

        # Default strength and alert level for all rules. This will make the scan look for all other alerts, not just the specified ones - so the results will be quite noisy.
        self.default_threshold = default_threshold
        self.default_strength = default_strength

    def rendered(
            self,
            active_scan_rule_ids: Union[list, str] = None,
            max_rule_duration: int = 10,
            max_scan_duration: int = 10
    ):
        # Create objects that manage config data
        active_scan = ActiveScanConfig(active_scan_rule_ids=active_scan_rule_ids)
        report_config = ReportConfig()
        # Get the template contents
        template_contents = dict(
            # env section
            target_url=self.target_url,
            include_paths=self.include_paths,
            exclude_paths=self.exclude_paths,

            # addOns section
            plugins_to_update=self.add_ons.plugins_to_update,
            update_plugins=self.add_ons.update_plugins_string,

            # activeScan section
            active_scan_rule_ids=active_scan.active_scan_rule_ids,
            alert_threshold=active_scan.alert_threshold,
            attack_strength=active_scan.attack_strength,
            max_rule_duration=max_rule_duration,
            max_scan_duration=max_scan_duration,
            # max_rule_duration=active_scan.max_rule_duration,
            # max_scan_duration=active_scan.max_scan_duration,
            default_strength=active_scan.default_strength,
            default_threshold=active_scan.default_threshold,

            # report section
            template_id=report_config.template_id,
            report_dir=report_config.report_dir,
        )
        # Render the template
        template_path = os.path.join(os.path.dirname(__file__), "templates")
        env = Environment(loader=FileSystemLoader(template_path), lstrip_blocks=True)  # nosec
        template = env.get_template("template.yaml.j2")
        result = template.render(t=template_contents)
        return result


def get_rule_id_list(rule_ids: Union[list, str]) -> Union[list, None]:
    """Get a list of rule IDs, given a string, a comma separated string, or a list."""
    if isinstance(rule_ids, str):
        if "," in rule_ids:
            rule_ids = rule_ids.split(",")
        else:
            rule_ids = [rule_ids]
        return rule_ids
    elif isinstance(rule_ids, list):
        return rule_ids
    elif isinstance(rule_ids, type(None)):
        return None
